@@ -4,16 +4,18 @@ from jinja2 import Environment, FileSystemLoader
 import os
 import subprocess
 import boto3
+import shutil
 
 
 class Deployer(object):
-    def __init__(self, name, dir, docker=False, environment="prod"):
+    def __init__(self, name, dir, docker=False, environment="prod", local=False):
         self.dir = dir
         script_dir = os.path.dirname(os.path.realpath(__file__))
         self.jinja = Environment(loader=FileSystemLoader(script_dir))
         self.project_name = name
         self.environment = environment
         self.docker = docker
+        self.local = local
 
     def _get_env_vars(self):
         # Getting environment variables.
@@ -30,7 +32,13 @@ class Deployer(object):
                 )
         return env_vars
 
-    def _build_template(self):
+    def _build_template(self, output_dir=None):
+        # Appending the current path to grab any other files.
+        os.sys.path.append(self.dir)
+
+        # Checking custom output directory.
+        output_file = f"{output_dir}/bld.yml" if output_dir else "bld.yml"
+
         # Running files to create subclasses.
         # TODO: Change these from hardcoded to dynamic.
         exec(open("function.py").read())
@@ -68,10 +76,33 @@ class Deployer(object):
             dynamo_tables=[],
             subdomain=self.project_name,
         )
-        f = open("template.yml", "w")
+        f = open(output_file, "w")
         f.write(rendered)
         f.close()
 
+    def _local_build(self):
+        build_path = f"{self.dir}/.aws-sam/build"
+        if os.path.exists(build_path):
+            shutil.rmtree(build_path)
+
+        os.mkdir(build_path)
+        pip_cmd = ["pip", "install", "-r", "requirements.txt", "-t", build_path]
+        subprocess.run(pip_cmd, cwd=self.dir)
+        shutil.copy("bld.yml", build_path)
+
+    def _sam_build(self):
+        # Run the SAM CLI to build and deploy.
+        sam_build = ["sam", "build", "--debug", "--template-file", "bld.yml"]
+        if self.docker:
+            sam_build.append("--use-container")
+        subprocess.run(sam_build, cwd=self.dir, check=True)
+
+    def _build(self):
+        if self.local:
+            self._local_build()
+        else:
+            self._sam_build()
+
     def deploy(self):
         self._build_template()
 
@@ -82,48 +113,48 @@ class Deployer(object):
             ACL="private", Bucket=f"{self.project_name}-{self.environment}"
         )
 
-        # Run the SAM CLI to build and deploy.
-        sam_build = ["sam", "build", "--debug"]
-        if self.docker:
-            sam_build.append("--use-container")
-        subprocess.run(sam_build, cwd=self.dir, check=True)
-        subprocess.run(
-            [
-                "sam",
-                "package",
-                "--s3-bucket",
-                f"{self.project_name}-{self.environment}",
-            ],
-            cwd=self.dir,
-        )
+        self._build()
+
+        # subprocess.run(
+        #     [
+        #         "sam",
+        #         "package",
+        #         "--s3-bucket",
+        #         f"{self.project_name}-{self.environment}",
+        #         "--template-file",
+        #         "bld.yml",
+        #     ],
+        #     cwd=self.dir,
+        # )
         env_vars = self._get_env_vars()
         overrides = [
             f"ParameterKey={x['alpha_name']},ParameterValue={x['value']}"
             for x in env_vars
         ]
-        subprocess.run(
-            [
-                "sam",
-                "deploy",
-                "--stack-name",
-                f"{self.project_name}-{self.environment}",
-                "--capabilities",
-                "CAPABILITY_NAMED_IAM",
-                "--s3-bucket",
-                f"{self.project_name}-{self.environment}",
-                "--parameter-overrides",
-                f"ENVIRONMENT={self.environment}",
-                ",".join(overrides),
-            ],
-            cwd=self.dir,
-            check=True,
-        )
+        sam_deploy = [
+            "sam",
+            "deploy",
+            "--stack-name",
+            f"{self.project_name}-{self.environment}",
+            "--capabilities",
+            "CAPABILITY_NAMED_IAM",
+            "--s3-bucket",
+            f"{self.project_name}-{self.environment}",
+            "--template-file",
+            "bld.yml",
+            "--parameter-overrides",
+            f"ENVIRONMENT={self.environment}",
+        ]
+        if len(overrides) > 0:
+            sam_deploy.append(",".join(overrides))
+        subprocess.run(sam_deploy, cwd=self.dir, check=True)
 
         print("Deployed successfully.")
 
     def start_api(self):
         self._build_template()
-        subprocess.run(["sam", "build", "--use-container"], cwd=self.dir, check=True)
+        self._build()
+
         env_vars = self._get_env_vars()
         overrides = [f"{x['alpha_name']}={x['value']}" for x in env_vars]
         subprocess.run(
@@ -141,4 +172,5 @@ class Deployer(object):
 
     def invoke(self, function):
         self._build_template()
+        self._build()
         subprocess.run(["sam", "local", "invoke", function], cwd=self.dir, check=True)
